@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2005-2014 by Oleksandr Shneyder                         *
+*   Copyright (C) 2005-2015 by Oleksandr Shneyder                         *
 *   o.shneyder@phoca-gmbh.de                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -242,6 +242,10 @@ void HttpBrokerClient::slotSshUserAuthError(QString error)
 void HttpBrokerClient::getUserSessions()
 {
     QString brokerUser=config->brokerUser;
+    // Otherwise, after logout from the session, we will be connected by a previous user without a password by authid.
+    if (config->brokerAutologoff) {
+        nextAuthId=config->brokerUserId;
+    }
     x2goDebug<<"called getUserSessions: brokeruser: "<<brokerUser<<" authid: "<<nextAuthId;
     if(mainWindow->getUsePGPCard())
         brokerUser=mainWindow->getCardLogin();
@@ -251,8 +255,8 @@ void HttpBrokerClient::getUserSessions()
         QString req;
         QTextStream ( &req ) <<
                              "task=listsessions&"<<
-                             "user="<<brokerUser<<"&"<<
-                             "password="<<config->brokerPass<<"&"<<
+                             "user="<<QUrl::toPercentEncoding(brokerUser)<<"&"<<
+                             "password="<<QUrl::toPercentEncoding(config->brokerPass)<<"&"<<
                              "authid="<<nextAuthId;
 
         x2goDebug << "sending request: "<< req.toUtf8();
@@ -279,7 +283,7 @@ void HttpBrokerClient::getUserSessions()
 
 void HttpBrokerClient::selectUserSession(const QString& session)
 {
-    x2goDebug<<"called selectUserSessions";
+    x2goDebug<<"Called selectUserSession for session "<<session<<".";
     QString brokerUser=config->brokerUser;
     if(mainWindow->getUsePGPCard())
         brokerUser=mainWindow->getCardLogin();
@@ -290,8 +294,8 @@ void HttpBrokerClient::selectUserSession(const QString& session)
         QTextStream ( &req ) <<
                              "task=selectsession&"<<
                              "sid="<<session<<"&"<<
-                             "user="<<brokerUser<<"&"<<
-                             "password="<<config->brokerPass<<"&"<<
+                             "user="<<QUrl::toPercentEncoding(brokerUser)<<"&"<<
+                             "password="<<QUrl::toPercentEncoding(config->brokerPass)<<"&"<<
                              "authid="<<nextAuthId;
         x2goDebug << "sending request: "<< req.toUtf8();
         QNetworkRequest request(QUrl(config->brokerurl));
@@ -302,10 +306,10 @@ void HttpBrokerClient::selectUserSession(const QString& session)
     else
     {
         if (nextAuthId.length() > 0) {
-            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+nextAuthId+ " --task selectsession --sid \\\""+session+"\\\"",
+            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --authid "+nextAuthId+ " --task selectsession --sid \""+session+"\"",
                                             this,SLOT ( slotSelectSession(bool,QString,int)));
         } else {
-            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --task selectsession --sid \\\""+session+"\\\"",
+            sshConnection->executeCommand ( config->sshBrokerBin+" --user "+ brokerUser +" --task selectsession --sid \""+session+"\"",
                                             this,SLOT ( slotSelectSession(bool,QString,int)));
         }
     }
@@ -324,9 +328,9 @@ void HttpBrokerClient::changePassword(QString newPass)
         QString req;
         QTextStream ( &req ) <<
                              "task=setpass&"<<
-                             "newpass="<<newPass<<"&"<<
-                             "user="<<brokerUser<<"&"<<
-                             "password="<<config->brokerPass<<"&"<<
+                             "newpass="<<QUrl::toPercentEncoding(newPass)<<"&"<<
+                             "user="<<QUrl::toPercentEncoding(brokerUser)<<"&"<<
+                             "password="<<QUrl::toPercentEncoding(config->brokerPass)<<"&"<<
                              "authid="<<nextAuthId;
         x2goDebug << "sending request: "<< req.toUtf8();
         QNetworkRequest request(QUrl(config->brokerurl));
