@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2005-2014 by Oleksandr Shneyder                         *
+*   Copyright (C) 2005-2015 by Oleksandr Shneyder                         *
 *   o.shneyder@phoca-gmbh.de                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -592,6 +592,7 @@ private:
     bool startHidden;
     bool keepTrayIcon;
     bool hideFolderSharing;
+    bool brokerNoauthWithSessionUsername;
     bool defaultUseSound;
     bool defaultXinerama;
     bool cardStarted;
@@ -836,6 +837,9 @@ private:
     bool cyEntry;
 
     QString pulseDir;
+    QString pulseBaseDir;
+    QString pulseRuntimeDir;
+    QString pulseCookieArg;
     int pulsePort;
     int esdPort;
     bool maximizeProxyWin;
