@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2005-2014 by Oleksandr Shneyder                         *
+*   Copyright (C) 2005-2015 by Oleksandr Shneyder                         *
 *   o.shneyder@phoca-gmbh.de                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -48,8 +48,7 @@
 
 #define PROXYTUNNELPORT 44444
 
-#undef DEBUG
-// #define DEBUG
+#define DEBUG
 
 #undef SSH_DEBUG
 // #define SSH_DEBUG
@@ -104,10 +103,10 @@ void SshMasterConnection::parseKnownHosts()
         for (int i=0; i<bytes.count();)
         {
             int size=0;
-            //first 4 bytes are for size of data fild (big-endian)
+            //first 4 bytes are for size of data field (big-endian)
             for (int j=0; j<4; ++j)
             {
-                size+=((uchar)(bytes[i])) * pow(256,3-j);
+                size+=((uchar)(bytes[i])) * pow((float)256,3-j);
                 i++;
             }
             QByteArray data;
@@ -333,7 +332,7 @@ void SshMasterConnection::checkReverseTunnelConnections()
 
                 if ( ::connect ( sock, ( struct sockaddr * ) &address,sizeof ( address ) ) !=0 )
                 {
-                    QString errMsg=tr ( "can not connect to " ) +
+                    QString errMsg=tr ( "Cannot connect to " ) +
                                    req.localHost+":"+QString::number ( req.localPort );
 #ifdef DEBUG
                     x2goDebug<<errMsg<<endl;
@@ -480,7 +479,7 @@ void SshMasterConnection::run()
 #endif
         if ( ssh_init() !=0 )
         {
-            QString err=tr ( "Can not initialize libssh" );
+            QString err=tr ( "Cannot initialize libssh" );
 #ifdef DEBUG
             x2goDebug<<err<<endl;
 #endif
@@ -508,7 +507,7 @@ void SshMasterConnection::run()
     my_ssh_session = ssh_new();
     if ( my_ssh_session == NULL )
     {
-        QString err=tr ( "Can not create ssh session" );
+        QString err=tr ( "Cannot create ssh session" );
 #ifdef DEBUG
         x2goDebug<<err<<endl;
 #endif
@@ -544,7 +543,7 @@ void SshMasterConnection::run()
         proxysocket = tcpProxySocket->socketDescriptor();
         if (!tcpProxySocket->waitForConnected(30000))
         {
-            QString message=tr ( "Can not connect to proxy server" );
+            QString message=tr ( "Cannot connect to proxy server" );
 #ifdef DEBUG
             x2goDebug<<message<<endl;
 #endif
@@ -574,7 +573,7 @@ void SshMasterConnection::run()
             return;
         }
         QString err=ssh_get_error ( my_ssh_session );
-        QString message=tr ( "Can not connect to " ) +host+":"+QString::number ( port );
+        QString message=tr ( "Cannot connect to " ) +host+":"+QString::number ( port );
 #ifdef DEBUG
         x2goDebug<<message<<" - "<<err;
 #endif
@@ -1140,14 +1139,14 @@ bool SshMasterConnection::userAuthKrb()
     QUuid uuid = QUuid::createUuid();
     QString uuidStr = uuid.toString().mid(1, 36).toLower();
 
-    QString shcmd = " echo X2GODATABEGIN:" + uuidStr + "; whoami; echo X2GODATAEND:" + uuidStr ;
+    QString shcmd = "sh -c 'echo X2GODATABEGIN:" + uuidStr + "; whoami; echo X2GODATAEND:" + uuidStr +";'";
 
 #ifdef Q_OS_WIN
     sshCmd="plink -batch "+user+"@"+host+" -P "+
-           QString::number(port)+ shcmd;
+           QString::number(port)+ " "+shcmd;
 #else
     sshCmd="ssh -o GSSApiAuthentication=yes "+user+"@"+host+" -p "+
-           QString::number(port)+ " -o PasswordAuthentication=no "+shcmd;
+           QString::number(port)+ " -o PasswordAuthentication=no -o PubkeyAuthentication=no "+shcmd;
 #endif
 
 #ifdef DEBUG
@@ -1293,7 +1292,7 @@ void SshMasterConnection::copy()
         QFile file ( copyRequests[i].src );
         if ( !file.open ( QIODevice::ReadOnly ) )
         {
-            QString errMsg=tr ( "Can not open file " ) +copyRequests[i].src;
+            QString errMsg=tr ( "Cannot open file " ) +copyRequests[i].src;
             emit copyErr ( copyRequests[i].creator, errMsg, "" );
             copyRequests.removeAt ( i );
             ssh_scp_close ( scp );
@@ -1305,7 +1304,7 @@ void SshMasterConnection::copy()
         rc=ssh_scp_push_file ( scp,dstFile.toAscii(),arr.size(), 0600 );
         if ( rc != SSH_OK )
         {
-            QString errMsg=tr ( "Can not create remote file " ) +copyRequests[i].dst;
+            QString errMsg=tr ( "Cannot create remote file " ) +copyRequests[i].dst;
             QString serr=ssh_get_error ( my_ssh_session );
 #ifdef DEBUG
             x2goDebug<<errMsg<<" - "<<serr<<endl;
@@ -1320,7 +1319,7 @@ void SshMasterConnection::copy()
         if ( rc != SSH_OK )
         {
             QString serr=ssh_get_error ( my_ssh_session );
-            QString errMsg=tr ( "Can not write to remote file " ) +copyRequests[i].dst;
+            QString errMsg=tr ( "Cannot write to remote file " ) +copyRequests[i].dst;
 #ifdef DEBUG
             x2goDebug<<errMsg<<" - "<<serr<<endl;
 #endif
@@ -1539,7 +1538,7 @@ void SshMasterConnection::channelLoop()
             if ( rez==SSH_EOF )
             {
 #ifdef DEBUG
-                x2goDebug<<"EOF ON CHANNEL "<<channel<<endl;
+                x2goDebug<<"EOF ON CHANNEL "<<channel<<" (SshProcess: "<<channelConnections[i].creator->pid<<")"<<endl;
 #endif
                 //////Finished////////
                 finalize ( i );
@@ -1594,7 +1593,7 @@ void SshMasterConnection::channelLoop()
                 if ( channel_is_eof ( channel ) )
                 {
 #ifdef DEBUG
-                    x2goDebug<<"EOF ON CHANNEL "<<channel<<endl;
+                    x2goDebug<<"EOF ON CHANNEL "<<channel<<" (SshProcess: "<<channelConnections[i].creator->pid<<")"<<endl;
 #endif
                     //////Finished////////
                     finalize ( i );
@@ -1674,8 +1673,11 @@ void SshMasterConnection::finalize ( int item )
     {
 #ifndef Q_OS_WIN
         shutdown(tcpSocket, SHUT_RDWR);
-#endif
         close ( tcpSocket );
+#else
+        shutdown(tcpSocket, SD_BOTH);
+        closesocket ( tcpSocket );
+#endif
     }
     SshProcess* proc=channelConnections[item].creator;
     QString uuid=channelConnections[item].uuid;
