@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2005-2014 by Oleksandr Shneyder                         *
+*   Copyright (C) 2005-2015 by Oleksandr Shneyder                         *
 *   o.shneyder@phoca-gmbh.de                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -28,8 +28,8 @@
 #include <netinet/tcp.h>
 #endif
 
-#undef DEBUG
-// #define DEBUG
+// #undef DEBUG
+#define DEBUG
 
 #define KEEPALIVE_OPTION " -o ServerAliveInterval=60 "
 
@@ -186,11 +186,11 @@ void SshProcess::startNormal(const QString& cmd)
 
 //#ifdef DEBUG
 // ONLY UNCOMMENT FOR TESTING, MIGHT REVEAL PASSWORD WHEN command=RDP
-//    x2goDebug<<"executing remote command: "<<shcmd<<endl;
+    x2goDebug<<"executing remote command via SshProcess object ("<<pid<<"): "<<cmd<<endl;
 // #endif
     if(!masterCon->useKerberos())
     {
-        QString shcmd = "sh -c \"echo X2GODATABEGIN:" + uuidStr + "; PATH=/usr/local/bin:/usr/bin:/bin "+cmd+"; echo X2GODATAEND:" + uuidStr +"\";";
+        QString shcmd = "sh -c 'echo X2GODATABEGIN:" + uuidStr + "; PATH=/usr/local/bin:/usr/bin:/bin "+cmd+"; echo X2GODATAEND:" + uuidStr +";'";
         masterCon->addChannelConnection(this, uuidStr, shcmd);
         connect(masterCon,SIGNAL(stdOut(SshProcess*,QByteArray)),this,SLOT(slotStdOut(SshProcess*,QByteArray)));
         connect(masterCon,SIGNAL(channelClosed(SshProcess*,QString)), this,SLOT(slotChannelClosed(SshProcess*,QString)));
@@ -198,7 +198,7 @@ void SshProcess::startNormal(const QString& cmd)
     else
     {
         QString host=masterCon->getHost();
-        QString shcmd = "echo X2GODATABEGIN:" + uuidStr + "; PATH=/usr/local/bin:/usr/bin:/bin "+cmd+"; echo X2GODATAEND:" + uuidStr;
+        QString shcmd = "sh -c 'echo X2GODATABEGIN:" + uuidStr + "; PATH=/usr/local/bin:/usr/bin:/bin "+cmd+"; echo X2GODATAEND:" + uuidStr +";'";
         proc=new QProcess(this);
 #ifdef Q_OS_WIN
         if(masterCon->get_kerberosDelegation())
@@ -213,10 +213,12 @@ void SshProcess::startNormal(const QString& cmd)
         {
             krbDelegOption=" -K ";
         }
-        QString sshString=QString::null+"ssh"+ KEEPALIVE_OPTION +krbDelegOption+" -o GSSApiAuthentication=yes -o PasswordAuthentication=no -p "+
+        QString sshString=QString::null+"ssh"+ KEEPALIVE_OPTION +krbDelegOption+" -o GSSApiAuthentication=yes -o PasswordAuthentication=no -o PubkeyAuthentication=no -p "+
 #endif
                           QString::number(masterCon->getPort())+" -l "+
                           masterCon->getUser()+" "+ host +  " \""+shcmd+"\"";
+
+        x2goDebug<<"evoking ssh command via SshProcess object ("<<pid<<"): "<<sshString<<endl;
         procUuid=uuidStr;
         proc->start(sshString);
 
@@ -224,7 +226,7 @@ void SshProcess::startNormal(const QString& cmd)
         {
             stdErrString=proc->errorString();
 #ifdef DEBUG
-            x2goDebug<<"ssh start failed:" <<stdErrString<<endl;
+            //x2goDebug<<"ssh start failed:" <<stdErrString<<endl;
 #endif
             slotChannelClosed(this, uuidStr);
             return;
@@ -239,6 +241,8 @@ void SshProcess::startNormal(const QString& cmd)
 
 void SshProcess::start_cp(QString src, QString dst)
 {
+    x2goDebug<<"copying file via SshProcess object ("<<pid<<"): "<<src<<" -> "<<dst<<endl;
+
     scpSource=src;
     if(!masterCon->useKerberos())
     {
@@ -258,7 +262,7 @@ void SshProcess::start_cp(QString src, QString dst)
 
         QString sshString="pscp -batch -P "+
 #else
-        QString sshString="scp -o GSSApiAuthentication=yes -o PasswordAuthentication=no -P "+
+        QString sshString="scp -o GSSApiAuthentication=yes -o PasswordAuthentication=no -o PubkeyAuthentication=no -P "+
 #endif
                           QString::number(masterCon->getPort())+" "+src+" "+
                           masterCon->getUser()+"@"+ masterCon->getHost()+":"+dst;
@@ -287,6 +291,8 @@ void SshProcess::start_cp(QString src, QString dst)
 void SshProcess::startTunnel(const QString& forwardHost, uint forwardPort, const QString& localHost,
                              uint localPort, bool reverse)
 {
+    x2goDebug<<"Starting tunnel via SshProcess object ("<<pid<<"): "<<forwardHost<<":"<<forwardPort<<" -> "<<localHost<<":"<<localPort<<endl;
+
     tunnel=true;
     tunnelOkEmited=false;
     if(!masterCon->useKerberos())
@@ -304,7 +310,7 @@ void SshProcess::startTunnel(const QString& forwardHost, uint forwardPort, const
 #ifdef Q_OS_WIN
         QString sshString="plink -batch -P "+
 #else
-        QString sshString=QString::null+"ssh"+ KEEPALIVE_OPTION +"-o GSSApiAuthentication=yes -o PasswordAuthentication=no -p "+
+        QString sshString=QString::null+"ssh"+ KEEPALIVE_OPTION +"-o GSSApiAuthentication=yes -o PasswordAuthentication=no -o PubkeyAuthentication=no -p "+
 #endif
                           QString::number(masterCon->getPort())+" "+
                           masterCon->getUser()+"@"+
@@ -373,11 +379,13 @@ void SshProcess::slotIOerr(SshProcess* creator, QString message, QString sshSess
 {
     if (creator!=this)
         return;
+    if (sshSessionErr.length())
+        sshSessionErr = " - "+sshSessionErr;
 #ifdef DEBUG
-    x2goDebug<<"io error:"<<message<<" - "<<sshSessionErr<<endl;
+    x2goDebug<<"I/O error: "<<message<<sshSessionErr<<" ("<<pid<<")."<<endl;
 #endif
     normalExited=false;
-    abortString=message+" - "+sshSessionErr;
+    abortString="I/O error: "+message+sshSessionErr;
 }
 
 void SshProcess::slotCopyErr(SshProcess* creator, QString message, QString sshSessionErr)
@@ -438,7 +446,7 @@ void SshProcess::slotChannelClosed(SshProcess* creator, QString uuid)
         }
     }
 #ifdef DEBUG
-    x2goDebug<<"ssh finished:"<<normalExited<<" - "<<output<<uuid<<endl;
+    x2goDebug<<"ssh finished: "<<normalExited<<" - "<<output<<" ("<<pid<<")."<<endl;
 #endif
     emit sshFinished(normalExited, output, pid);
 }
